#import "Average.h"
//#define DEBUG

#ifdef DEBUG
#define LED1 2
#define LED2 3
#define SWITCHMODE 4
#define SWITCHPCB 5
#define PTC 0
#define HEAT 6
#define debug(x) Serial.print(x)
#define debugln(x) Serial.println(x)
#else
#define LED1 0
#define LED2 1
#define HEAT 2
#define SWITCHMODE 3
#define SWITCHPCB 5
#define PTC 2
#define debug(x)
#define debugln(x)
#endif
#define MINTEMPPCB 20
#define MINTEMPLOW 100
#define MINTEMPHIGH 80

Average average;
unsigned long  avgTimer;

int minTemp = 13;

int ctr = 0;
unsigned long hotTimer;
int shiftOn = 10000;
int shiftOff = 13000;
int lastTemp = 1000;
bool heatSwitch = true;
unsigned long  blinkTimer;
int blinkFreq;
bool blinkState = false;
bool hot = false;

enum Phase { heating, constanting, cooling };
Phase phase = heating;
enum HeatState { heatingLow, heatingHigh, heatingPCB };
HeatState heatState = heatingLow;

void setup() {
#ifdef DEBUG
  Serial.begin(115200);
  debugln("Start");
#endif

  pinMode(HEAT, OUTPUT);
  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(SWITCHMODE, INPUT);
  pinMode(SWITCHPCB, INPUT);
  digitalWrite(LED1, HIGH);
  digitalWrite(LED2, HIGH);
  digitalWrite(HEAT, HIGH);
  blinkTimer = millis();
  avgTimer = millis();
  for (int i = 0; i < SAMPLES; i++) average.sample(analogRead(PTC));

}

void signalLeds() { // flash both LEDs synchronously 3 times
  int mode = 0;
  for (int i = 0; i < 6; i++) {
    digitalWrite(LED1, mode);
    digitalWrite(LED2, mode);
    mode = 1 - mode;
    delay(100 + mode * 100);
  }
}

void loop() {
  if (millis() > avgTimer) {
    average.sample(analogRead(PTC));
    avgTimer += 500; // measure every 500ms
  }

  int temp = average.average(); // just look at the average temperature (direct reading has ripples)
  int minTemp = 999; // will be set conditionally below
  HeatState nextHeatState;

  switch (phase) {
    case heating:
      if (heatState != heatingPCB) { // standard mode (PCB switch has not been pressed
        if (digitalRead(SWITCHPCB)) { // high if not pressed
          nextHeatState = digitalRead(SWITCHMODE) == LOW ? heatingLow : heatingHigh; // mode switch sets different heat values
          if (nextHeatState != heatState) {
            debug("switch ");
            debugln(nextHeatState);
            signalLeds();
            heatState = nextHeatState;
          }
        } else {
          heatState = heatingPCB;
          signalLeds();
          debugln("->PCB");
        }
      }
      switch (heatState) {
        case heatingLow:
          minTemp = MINTEMPLOW;
          break;

        case heatingHigh:
          minTemp = MINTEMPHIGH;
          break;

        case heatingPCB:
          minTemp = MINTEMPPCB;
          break;
      }
      if (temp > minTemp) break;
      debugln("Finished heat");
      heatSwitch = false;
      hotTimer = millis();
      phase = constanting;
      break;

    case constanting:
      if (millis() > hotTimer + 60000) {
        heatSwitch = false;
        debugln("Stop heat");
        phase = cooling;
        digitalWrite(LED1, LOW);
        break;
      }

      heatSwitch = (temp > minTemp);
      digitalWrite(LED1, ((millis() - hotTimer) / 1000L) % 2);
      break;

    case cooling:
      break;
  }

  if (millis() > blinkTimer) {
    blinkState = !blinkState;
    digitalWrite(LED2, blinkState);
    blinkTimer += (temp - minTemp) * 5;
  }

  digitalWrite(HEAT, heatSwitch);

#ifdef DEBUG
  if (Serial.available()) {
    Serial.read();
    Serial.print(heatState);
    Serial.print("/");
    Serial.print(phase);
    Serial.print("/");
    Serial.println(temp);
  }
#endif
}
